//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef INST_SECTION_HXX
# define INST_SECTION_HXX

# include "inst/section.hh"

# include "misc/contract.hh"

namespace inst
{

  //
  inline
  Section::Section(const std::string &name):
    name_(name)
  {
  }

  inline bool
  Section::has_label(const misc::unique_string &label) const
  {
    Label dummy(label, 0);

    label_set_type::const_iterator it = sorted_labels_.find(&dummy);
    return it != sorted_labels_.end();
  }
  inline int
  Section::get_offset(const misc::unique_string &label) const
  {
    Label dummy(label, 0);

    label_set_type::const_iterator it = sorted_labels_.find(&dummy);
    assertion(it != sorted_labels_.end());
    return (*it)->get_offset();
  }

  //
  inline std::ostream &
  operator<<(std::ostream &ostr, const Section &s)
  {
    s.write(ostr);
    return ostr;
  }

} // namespace inst

#endif // !INST_SECTION_HXX
