//
// escape.hh: escaping special characters for output
// Copyright (C) 2003, 2004 Akim Demaille <akim@epita.fr> and 
//                          Benoit Perrot <benoit@lrde.epita.fr>
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// It is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef MISC_ESCAPE_HH
# define MISC_ESCAPE_HH

# include <ios>
# include <iomanip>
# include <iostream>
# include <string>
# include <cctype>
# include <locale>

class escape
{
public:
  escape (const std::string& s) : s_ (s)
  {}

  const std::string& s_;
  bool print_delim_;
};

inline std::ostream&
operator<<(std::ostream& o, const escape& e)
{
  static std::locale locale ("");
  std::ios_base::fmtflags flags = o.flags (std::ios_base::hex);
  for (std::string::const_iterator p = e.s_.begin (); p != e.s_.end (); ++p)
    switch (*p)
      {
      case '\a': o << "\\a"; break;
      case '\b': o << "\\b"; break;
      case '\f': o << "\\f"; break;
      case '\n': o << "\\n"; break;
      case '\r': o << "\\r"; break;
      case '\t': o << "\\t"; break;

      case '\\': o << "\\\\"; break;
      case '\"': o << "\\\""; break;
      default:
	if (std::isprint (*p, locale))
	  o << *p;
	else
	  o << "\\x"
	    << std::setfill ('0') << std::setw (2) << (int) (unsigned char) *p;
      }
  o.flags (flags);
  return o;
}

#endif // !MISC_ESCAPE_HH
