//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2004 Akim Demaille <akim@epita.fr> and
//		      Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//

#include <iostream>
#include <sstream>

#include "task/int_task.hh"

namespace task
{

  IntTask::IntTask(int &var, int min, int max,
		   const std::string& option,
		   const std::string& module_name,
		   const std::string& desc,
		   const std::string& deps):
    Task(option, module_name, desc, deps, true),
    var_(var), min_(min), max_(max)
  {}

  bool
  IntTask::set_value(const std::string &value) const
  {
    std::istringstream iss(value);
    iss >> value_;

    // FIXME: print program_name!
    if (iss.fail())
      std::cerr << ": expected an integer: " << value << std::endl;
    else if (value_ < min_ || max_ < value_)
      std::cerr << ": invalid integer: " << value_ << std::endl;
    else
      return true;

    return false;
  }

  void
  IntTask::execute(void) const
  {
    var_ = value_;
  }

} //namespace task
