//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef VM_CP0_HXX
# define VM_CP0_HXX

# include "vm/cp0.hh"
# include "vm/system_library.hh"

# include "inst/register.hh"

# include "common.hh"

namespace vm
{

  //
  inline
  Cp0::Cp0(Cpu &cpu):
    cpu_(cpu),
    system_library_(0)
  {
    reset();
  }

  inline void
  Cp0::reset()
  {
    // Initialize registers to 0.
    for (unsigned i = 0; i < 32; ++i)
      registers[i] = 0;

    fatal_exception_ = false;
  }

  inline void
  Cp0::set_system_library(SystemLibrary *l)
  {
    system_library_ = l;
  }

  //
  inline void
  Cp0::set_count(Cp0::register_type r)
  {
    registers[count] = r;
  }
  inline Cp0::register_type
  Cp0::get_count() const
  {
    return registers[count];
  }

  inline void
  Cp0::set_register(const inst::Register &reg, Cp0::register_type r)
  {
    precondition(reg.get_kind() == inst::Register::generic);
    registers[reg.get_index()] = r;
  }
  inline Cp0::register_type
  Cp0::get_register(const inst::Register &reg) const
  {
    precondition(reg.get_kind() == inst::Register::generic);
    return registers[reg.get_index()];
  }
  inline Cp0::register_type
  Cp0::get_register(kind_type k) const
  {
    return registers[k];
  }

  //
  inline void
  Cp0::raise_overflow()
  {
    registers[cause] = overflow;
    std::cerr << "Runtime Exception: Overflow" << std::endl;
    fatal_exception_ = true;
  }

  inline void
  Cp0::raise_addr_load()
  {
    registers[cause] = addr_load;
    std::cerr << "Runtime Exception: Address error on load" << std::endl;
    fatal_exception_ = true;
  }
  inline void
  Cp0::raise_addr_store()
  {
    registers[cause] = addr_store;
    std::cerr << "Runtime Exception: Address error on store" << std::endl;
    fatal_exception_ = true;
  }

  inline void
  Cp0::raise_syscall()
  {
    precondition(system_library_);
    system_library_->invoke(cpu_);
  }

  //
  inline void
  Cp0::set_fatal_exception()
  {
    fatal_exception_ = true;
  }

  inline bool
  Cp0::fatal_exception() const
  {
    return fatal_exception_;
  }

} // namespace vm

#endif // !VM_CP0_HXX
