//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#include "common.hh"

#include "vm/spim_system_library.hh"
#include "vm/cpu.hh"
#include "vm/cp0.hh"
#include "vm/mmu.hh"

namespace vm
{

  SpimSystemLibrary::~SpimSystemLibrary()
  {
  }

  void
  SpimSystemLibrary::invoke(Cpu &cpu)
  {
    Mmu &mmu = cpu.get_mmu();

    switch (cpu.get_register(Cpu::v0))
      {
	// print_int (integer: $a0)
      case 1:
	ostr_ << cpu.get_register(Cpu::a0);
	break;

	// print_string (buffer: $a0)
      case 4:
	for (int i = cpu.get_register(Cpu::a0); true; ++i)
	  {
	    char b = mmu.data_load_byte(i);
	    if (b == 0)
	      break;
	    ostr_ << b;
	  }
	break;

	// read_string (buffer: $a0, length: $a1)
      case 8:
	{
	  int i = 0;
	  int c = 0;
	  for (; (i < cpu.get_register(Cpu::a1) - 1)
		 && (c != '\n') && (c != '\r');
	       ++i)
	    {
	      c = istr_.get();
	      if (istr_.eof())
		break;
	      mmu.data_store_byte(cpu.get_register(Cpu::a0) + i, c);
	    }
	  mmu.data_store_byte(cpu.get_register(Cpu::a0) + i, 0);
	}
	break;

	// sbrk (size: $a0)
      case 9:
	cpu.set_register(Cpu::v0, mmu.data_sbrk(cpu.get_register(Cpu::a0)));
	break;

	// exit (status : $a0)
      case 10:
	{
	  cpu.get_cp0().set_fatal_exception();
	  if (!exit_status)
	    exit_status = (exit_type) cpu.get_register(Cpu::a0);
	}
	break;

	// print_err (buffer: $a0)
      case 15:
	for (int i = cpu.get_register(Cpu::a0); true; ++i)
	  {
	    char b = mmu.data_load_byte(i);
	    if (b == 0)
	      break;
	    std::cerr << b;
	  }
	break;

      default:
	assertion(!"syscall: Not implemented yet");
      };
    ostr_.flush();
  }

} // namespace vm
